@@ -2,7 +2,7 @@
 * socket.cpp: socket management
 *-------------------------------------------------------------------------------
 * (c)1999-2001 VideoLAN
-* $Id: socket.cpp,v 1.2 2001/10/16 12:24:39 bozo Exp $
+* $Id: socket.cpp,v 1.11 2002/04/27 01:15:57 asmax Exp $
 *
 * Authors: Benoit Steiner <benny@via.ecp.fr>
 *          Arnaud de Bossoreille de Ribou <bozo@via.ecp.fr>
@@ -171,14 +171,14 @@ void C_SocketBuff::AssignSlot(unsigned int iSlot, byte* pBuff,
   m_pBuffers[iSlot].iov_base = (bufftype*)pBuff;
   m_pBuffers[iSlot].iov_len = iBuffSize;
 #elif defined WIN32
-  ASSERT(false);
+  m_pBuffers[iSlot].buf = (bufftype*)pBuff;
+  m_pBuffers[iSlot].len = iBuffSize;
 #endif
 }
 
 
 
 
-
 //------------------------------------------------------------------------------
 // 
 //------------------------------------------------------------------------------
@@ -192,7 +192,7 @@ void C_SocketBuff::SetSlotBuff(unsigned int iSlot, byte* pBuff)
 #ifdef STRUCT_IOVEC_IN_SYS_UIO_H
   m_pBuffers[iSlot].iov_base = (bufftype*)pBuff;
 #elif defined WIN32
-  ASSERT(false);
+  m_pBuffers[iSlot].buf = (bufftype*)pBuff;
 #endif
 }
 
@@ -211,14 +211,13 @@ void C_SocketBuff::SetSlotSize(unsigned int iSlot, unsigned int iBuffSize)
 #ifdef STRUCT_IOVEC_IN_SYS_UIO_H
   m_pBuffers[iSlot].iov_len = iBuffSize;
 #elif defined WIN32
-  ASSERT(false);
+  m_pBuffers[iSlot].len = iBuffSize;
 #endif
 }
 
 
-
 //******************************************************************************
-// C_InetAddr class
+// C_Inet4Addr class
 //******************************************************************************
 // 
 //******************************************************************************
@@ -228,10 +227,10 @@ void C_SocketBuff::SetSlotSize(unsigned int iSlot, unsigned int iBuffSize)
 //------------------------------------------------------------------------------
 //
 //------------------------------------------------------------------------------
-void C_InetAddr::Build(const C_String& strHost, const C_String& strPort)
+void C_Inet4Addr::Build(const C_String& strHost, const C_String& strPort)
 {
   // Init the structure
-  memset(&m_sInetAddr, 0, sizeof(struct sockaddr_in));
+  memset(&m_sInetAddr, 0, sizeof(m_sInetAddr));
   m_sInetAddr.sin_family = AF_INET;
 
   // Set the port field
@@ -251,7 +250,7 @@ void C_InetAddr::Build(const C_String& strHost, const C_String& strPort)
     }
     else
       m_sInetAddr.sin_port = pPort->s_port;
-  } 
+  }
 
   // Set the address field
   // Assume the address is given in dotted-decimal format and try to convert
@@ -271,6 +270,50 @@ void C_InetAddr::Build(const C_String& strHost, const C_String& strPort)
 }
 
 
+//******************************************************************************
+// C_Inet4Addr class
+//******************************************************************************
+// 
+//******************************************************************************
+
+//------------------------------------------------------------------------------
+//
+//------------------------------------------------------------------------------
+//
+//------------------------------------------------------------------------------
+#ifdef HAVE_IPV6
+void C_Inet6Addr::Build(const C_String& strHost, const C_String& strPort,
+                        int iType)
+{
+  struct addrinfo sHints;
+  sHints.ai_flags = 0;
+  sHints.ai_family = AF_INET6;
+  sHints.ai_socktype = iType;
+  sHints.ai_protocol = 0;
+  sHints.ai_addrlen = 0;
+  sHints.ai_addr = NULL;
+  sHints.ai_canonname = NULL;
+  sHints.ai_next = NULL;
+
+  struct addrinfo *pRes = NULL;
+
+  int iRc = getaddrinfo(strHost.GetString(), strPort.GetString(),
+                        &sHints, &pRes);
+
+  if(!iRc)
+  {
+    ASSERT(pRes->ai_family == AF_INET6);
+    m_sInetAddr = *(sockaddr_in6*)(pRes->ai_addr);
+    freeaddrinfo(pRes);
+  }
+  else
+  {
+    ASSERT(!pRes);
+    throw E_InetAddr(C_String("Inet6 address error: ") + gai_strerror(iRc));
+  }
+}
+#endif
+
 
 //******************************************************************************
 // C_Socket class
@@ -285,6 +328,7 @@ void C_InetAddr::Build(const C_String& strHost, const C_String& strPort)
 //------------------------------------------------------------------------------
 C_Socket::C_Socket()
 {
+  m_iDomain = AF_UNSPEC;
   ZERO(m_hSocket);
 }
 
@@ -293,9 +337,12 @@ C_Socket::C_Socket()
 //------------------------------------------------------------------------------
 //
 //------------------------------------------------------------------------------
-void C_Socket::Open(int iType, int iProtocol/* = 0*/)
+void C_Socket::Open(int iDomain, int iType, int iProtocol/* = 0*/)
 {
-  m_hSocket = socket(AF_INET, iType, iProtocol);
+  m_iDomain = iDomain;
+  m_iType = iType;
+
+  m_hSocket = socket(m_iDomain, iType, iProtocol);
 
   if(m_hSocket < 0)
   {
@@ -359,19 +406,50 @@ void C_Socket::GetOption(int iLevel, int iOpt, void* pOptVal, int* pOptLen)
 void C_Socket::Bind(const C_String& strIntf, const C_String& strPort)
 {
   // Try to resolve the given address
-  C_InetAddr cIntfAddr;
-  try
+  struct sockaddr* pAddr;
+  socklen iAddrLen;
+
+  if(m_iDomain == AF_INET)
   {
-    cIntfAddr.Build(strIntf, strPort);
+    C_Inet4Addr cIntfAddr;
+
+    try
+    {
+      cIntfAddr.Build(strIntf, strPort);
+    }
+    catch(E_InetAddr e)
+    {
+      throw E_Socket("Unable to bind to " + strIntf + " on port " + strPort, e);
+    }
+
+    pAddr = (struct sockaddr*)cIntfAddr.GetInetAddr();
+    iAddrLen = sizeof(*cIntfAddr.GetInetAddr());
   }
-  catch(E_InetAddr e)
+#ifdef HAVE_IPV6
+  else if(m_iDomain == AF_INET6)
   {
-    throw E_Socket("Unable to bind to " + strIntf + " on port " + strPort, e);
+    C_Inet6Addr cIntfAddr;
+
+    try
+    {
+      cIntfAddr.Build(strIntf, strPort, m_iType);
+    }
+    catch(E_InetAddr e)
+    {
+      throw E_Socket("Unable to bind to " + strIntf + " on port " + strPort, e);
+    }
+
+    pAddr = (struct sockaddr*)cIntfAddr.GetInetAddr();
+    iAddrLen = sizeof(*cIntfAddr.GetInetAddr());
+  }
+#endif
+  else
+  {
+    ASSERT(false);
   }
   
   // Bind the socket to the local address
-  int iRc = bind(m_hSocket, (struct sockaddr*)cIntfAddr.GetInetAddr(),
-                 sizeof(*cIntfAddr.GetInetAddr()));
+  int iRc = bind(m_hSocket, pAddr, iAddrLen);
   if(iRc)
   {
     throw E_Socket("Unable to bind to " + strIntf + " on port " + strPort +
@@ -386,19 +464,52 @@ void C_Socket::Bind(const C_String& strIntf, const C_String& strPort)
 void C_Socket::Connect(const C_String& strPeer, const C_String& strPort)
 {
   // Try to resolve the given address
-  C_InetAddr cPeerAddr;
-  try
+  struct sockaddr* pAddr;
+  socklen iAddrLen;
+
+  if(m_iDomain == AF_INET)
   {
-    cPeerAddr.Build(strPeer, strPort);
+    C_Inet4Addr cPeerAddr;
+
+    try
+    {
+      cPeerAddr.Build(strPeer, strPort);
+    }
+    catch(E_InetAddr e)
+    {
+      throw E_Socket("Unable to connect to " + strPeer +
+                     " on port "+ strPort, e);
+    }
+
+    pAddr = (struct sockaddr*)cPeerAddr.GetInetAddr();
+    iAddrLen = sizeof(*cPeerAddr.GetInetAddr());
   }
-  catch(E_InetAddr e)
+#ifdef HAVE_IPV6
+  else if(m_iDomain == AF_INET6)
   {
-    throw E_Socket("Unable to connect to " + strPeer + " on port "+ strPort, e);
+    C_Inet6Addr cPeerAddr;
+
+    try
+    {
+      cPeerAddr.Build(strPeer, strPort, m_iType);
+    }
+    catch(E_InetAddr e)
+    {
+      throw E_Socket("Unable to connect to " + strPeer +
+                     " on port "+ strPort, e);
+    }
+
+    pAddr = (struct sockaddr*)cPeerAddr.GetInetAddr();
+    iAddrLen = sizeof(*cPeerAddr.GetInetAddr());
+  }
+#endif
+  else
+  {
+    ASSERT(false);
   }
 
   // Bind the socket to the local address
-  int iRc = connect(m_hSocket, (struct sockaddr*)cPeerAddr.GetInetAddr(),
-                    sizeof(*cPeerAddr.GetInetAddr()));
+  int iRc = connect(m_hSocket, pAddr, iAddrLen);
   if(iRc)
   {
     throw E_Socket("Unable to connect to " + strPeer + " on port " +
@@ -427,19 +538,40 @@ void C_Socket::Listen(int iBacklog)
 //------------------------------------------------------------------------------
 C_Socket* C_Socket::Accept()
 {
-  struct sockaddr_in sPeer;
-  socklen iAddrLen = sizeof(sPeer);
-  
-  int iRc = accept(m_hSocket, (struct sockaddr*)&sPeer, &iAddrLen);
+  int iRc = -1;
+
+  if(m_iDomain == AF_INET)
+  {
+    struct sockaddr_in sPeer;
+    socklen iAddrLen = sizeof(sPeer);
   
+    iRc = accept(m_hSocket, (struct sockaddr*)&sPeer, &iAddrLen);
+  }
+#ifdef HAVE_IPV6
+  else if(m_iDomain == AF_INET6)
+  {
+    struct sockaddr_in6 sPeer;
+    socklen iAddrLen = sizeof(sPeer);
+
+    iRc = accept(m_hSocket, (struct sockaddr*)&sPeer, &iAddrLen);
+  }
+#endif
+  else
+  {
+    ASSERT(false);
+  }
+
   C_Socket* pSocket = NULL;
 
   if(iRc < 0)
+  {
     throw E_Socket("Unable to accept connection from " + GetPeerName() +
                    ": "+GetErrorMsg(SYSTEM_NETWORK));
+  }
   else
   {
-    pSocket = new C_Socket();   
+    pSocket = new C_Socket();
+    pSocket->m_iDomain = m_iDomain;
     pSocket->m_hSocket = iRc;
   }
   
@@ -560,8 +692,8 @@ int C_Socket::Send(const C_SocketBuff& cBuffers, unsigned int iSlotCount)
   int iRc = writev(m_hSocket, pBuff, iSlotCount);
 
 #elif defined WIN32
-  ASSERT(false);
-  int iRc = GEN_ERR;
+  unsigned long count;
+  int iRc = WSASend(m_hSocket, pBuff, iSlotCount, &count, NULL, NULL, NULL);
 #endif
 
   if(iRc < 0)
@@ -585,10 +717,11 @@ int C_Socket::Send(const C_SocketBuff& cBuffers)
 //------------------------------------------------------------------------------
 //
 //------------------------------------------------------------------------------
-int C_Socket::WriteTo(C_InetAddr& cPeerAddr,
+int C_Socket::WriteTo(C_Inet4Addr& cPeerAddr,
                       const byte* pBuff, int iBuffLen, int iFlags)
 {
   ASSERT(pBuff);
+  ASSERT(m_iDomain == AF_INET);
 
   int iRc = sendto(m_hSocket, (bufftype*)pBuff, iBuffLen, iFlags,
                    (const sockaddr*)cPeerAddr.GetInetAddr(),
@@ -606,11 +739,36 @@ int C_Socket::WriteTo(C_InetAddr& cPeerAddr,
 //------------------------------------------------------------------------------
 //
 //------------------------------------------------------------------------------
-int C_Socket::SendTo(C_InetAddr& cPeerAddr,
+#ifdef HAVE_IPV6
+int C_Socket::WriteTo(C_Inet6Addr& cPeerAddr,
+                      const byte* pBuff, int iBuffLen, int iFlags)
+{
+  ASSERT(pBuff);
+  ASSERT(m_iDomain == AF_INET6);
+
+  int iRc = sendto(m_hSocket, (bufftype*)pBuff, iBuffLen, iFlags,
+                   (const sockaddr*)cPeerAddr.GetInetAddr(),
+                   sizeof(*cPeerAddr.GetInetAddr()));
+
+/*  if(iRc < 0)
+  {
+    throw E_Socket("Unable to send data: " + GetErrorMsg(SYSTEM_NETWORK));
+  }*/
+
+  return iRc;
+}
+#endif
+
+
+//------------------------------------------------------------------------------
+//
+//------------------------------------------------------------------------------
+int C_Socket::SendTo(C_Inet4Addr& cPeerAddr,
                      const C_SocketBuff& cBuffers, unsigned int iSlotCount)
 {
   // Further writing
   ASSERT(false);
+  ASSERT(m_iDomain == AF_INET);
   int iRc = GEN_ERR;
   return iRc;
 }
@@ -619,7 +777,7 @@ int C_Socket::SendTo(C_InetAddr& cPeerAddr,
 //------------------------------------------------------------------------------
 //
 //------------------------------------------------------------------------------
-int C_Socket::SendTo(C_InetAddr& cPeerAddr, const C_SocketBuff& cBuffers)
+int C_Socket::SendTo(C_Inet4Addr& cPeerAddr, const C_SocketBuff& cBuffers)
 {
   return SendTo(cPeerAddr, cBuffers, cBuffers.GetSlotCount());
 }
@@ -677,18 +835,48 @@ bool C_Socket::operator == (const C_Socket& cArg)
 C_String C_Socket::GetLocalName() const
 {
   C_String strLocalName("Unknown");
-  
-  struct sockaddr_in saIntf;
-  socklen iNameLen = sizeof(saIntf);
-  int iRc = getsockname(m_hSocket, (struct sockaddr*)&saIntf, &iNameLen);
-  ASSERT(iNameLen == sizeof(saIntf));
-  if(iRc)
+
+  if(m_iDomain == AF_INET)
   {
-    throw E_Socket("Unable to get name of the intf: " +
-                   GetErrorMsg(SYSTEM_NETWORK));
+    struct sockaddr_in saIntf;
+    socklen iNameLen = sizeof(saIntf);
+    int iRc = getsockname(m_hSocket, (struct sockaddr*)&saIntf, &iNameLen);
+    ASSERT(iNameLen == sizeof(saIntf));
+    if(iRc)
+    {
+      throw E_Socket("Unable to get name of the intf: " +
+                     GetErrorMsg(SYSTEM_NETWORK));
+    }
+    else
+      strLocalName = inet_ntoa(saIntf.sin_addr);
   }
+#ifdef HAVE_IPV6
+  else if(m_iDomain == AF_INET6)
+  {
+    struct sockaddr_in6 saIntf;
+    socklen iNameLen = sizeof(saIntf);
+    int iRc = getsockname(m_hSocket, (struct sockaddr*)&saIntf, &iNameLen);
+    ASSERT(iNameLen == sizeof(saIntf));
+    if(iRc)
+    {
+      throw E_Socket("Unable to get name of the intf: " +
+                     GetErrorMsg(SYSTEM_NETWORK));
+    }
+    else
+    {
+      char pszDst[40]; // 40 characters is enough for an IPv6 address
+      if(inet_ntop(AF_INET6, &saIntf.sin6_addr, pszDst, 40) == pszDst)
+        strLocalName = pszDst;
+      else if(errno != ENOSPC)
+        throw E_Socket("Unable to get name of the intf: " +
+                       GetErrorMsg(SYSTEM_NETWORK));
+    }
+  }
+#endif
   else
-    strLocalName = inet_ntoa(saIntf.sin_addr);
+  {
+    ASSERT(false);
+  }
   
   return strLocalName;
 }
@@ -701,17 +889,40 @@ C_String C_Socket::GetLocalPort() const
 {
   C_String strLocalPort("Unknown");
   
-  struct sockaddr_in saIntf;
-  socklen iNameLen = sizeof(saIntf);
-  int iRc = getsockname(m_hSocket, (struct sockaddr*)&saIntf, &iNameLen);
-  ASSERT(iNameLen == sizeof(saIntf));
-  if(iRc)
+  if(m_iDomain == AF_INET)
   {
-    throw E_Socket("Unable to get port of the intf: " +
-                   GetErrorMsg(SYSTEM_NETWORK));
+    struct sockaddr_in saIntf;
+    socklen iNameLen = sizeof(saIntf);
+    int iRc = getsockname(m_hSocket, (struct sockaddr*)&saIntf, &iNameLen);
+    ASSERT(iNameLen == sizeof(saIntf));
+    if(iRc)
+    {
+      throw E_Socket("Unable to get port of the intf: " +
+                     GetErrorMsg(SYSTEM_NETWORK));
+    }
+    else
+      strLocalPort = ntohs(saIntf.sin_port);
   }
+#ifdef HAVE_IPV6
+  else if(m_iDomain == AF_INET6)
+  {
+    struct sockaddr_in6 saIntf;
+    socklen iNameLen = sizeof(saIntf);
+    int iRc = getsockname(m_hSocket, (struct sockaddr*)&saIntf, &iNameLen);
+    ASSERT(iNameLen == sizeof(saIntf));
+    if(iRc)
+    {
+      throw E_Socket("Unable to get port of the intf: " +
+                     GetErrorMsg(SYSTEM_NETWORK));
+    }
+    else
+      strLocalPort = ntohs(saIntf.sin6_port);
+  }
+#endif
   else
-    strLocalPort = ntohs(saIntf.sin_port);
+  {
+    ASSERT(false);
+  }
   
   return strLocalPort;
 }
@@ -723,18 +934,48 @@ C_String C_Socket::GetLocalPort() const
 C_String C_Socket::GetPeerName() const
 {
   C_String strPeerName("Unknown");
-  
-  struct sockaddr_in saPeer;
-  socklen iNameLen = sizeof(saPeer);
-  int iRc = getpeername(m_hSocket, (struct sockaddr*)&saPeer, &iNameLen);
-  ASSERT(iNameLen == sizeof(saPeer));
-  if(iRc)
+
+  if(m_iDomain == AF_INET)
   {
-    throw E_Socket("Unable to get name of the peer: " +
-                   GetErrorMsg(SYSTEM_NETWORK));
+    struct sockaddr_in saPeer;
+    socklen iNameLen = sizeof(saPeer);
+    int iRc = getpeername(m_hSocket, (struct sockaddr*)&saPeer, &iNameLen);
+    ASSERT(iNameLen == sizeof(saPeer));
+    if(iRc)
+    {
+      throw E_Socket("Unable to get name of the peer: " +
+                     GetErrorMsg(SYSTEM_NETWORK));
+    }
+    else
+      strPeerName = inet_ntoa(saPeer.sin_addr);
   }
+#ifdef HAVE_IPV6
+  else if(m_iDomain == AF_INET6)
+  {
+    struct sockaddr_in6 saPeer;
+    socklen iNameLen = sizeof(saPeer);
+    int iRc = getpeername(m_hSocket, (struct sockaddr*)&saPeer, &iNameLen);
+    ASSERT(iNameLen == sizeof(saPeer));
+    if(iRc)
+    {
+      throw E_Socket("Unable to get name of the peer: " +
+                     GetErrorMsg(SYSTEM_NETWORK));
+    }
+    else
+    {
+      char pszDst[40]; // 40 characters is enough for an IPv6 address
+      if(inet_ntop(AF_INET6, &saPeer.sin6_addr, pszDst, 40) == pszDst)
+        strPeerName = pszDst;
+      else if(errno != ENOSPC)
+        throw E_Socket("Unable to get name of the intf: " +
+                       GetErrorMsg(SYSTEM_NETWORK));
+    }
+  }
+#endif
   else
-    strPeerName = inet_ntoa(saPeer.sin_addr);
+  {
+    ASSERT(false);
+  }
   
   return strPeerName;
 }
@@ -746,19 +987,42 @@ C_String C_Socket::GetPeerName() const
 C_String C_Socket::GetPeerPort() const
 {
   C_String strPeerPort("Unknown");
-  
-  struct sockaddr_in saPeer;
-  socklen iNameLen = sizeof(saPeer);
-  int iRc = getpeername(m_hSocket, (struct sockaddr*)&saPeer, &iNameLen);
-  ASSERT(iNameLen == sizeof(saPeer));
-  if(iRc)
+
+  if(m_iDomain == AF_INET)
   {
-    throw E_Socket("Unable to get port of the peer: " +
-                   GetErrorMsg(SYSTEM_NETWORK));
+    struct sockaddr_in saPeer;
+    socklen iNameLen = sizeof(saPeer);
+    int iRc = getpeername(m_hSocket, (struct sockaddr*)&saPeer, &iNameLen);
+    ASSERT(iNameLen == sizeof(saPeer));
+    if(iRc)
+    {
+      throw E_Socket("Unable to get port of the peer: " +
+                     GetErrorMsg(SYSTEM_NETWORK));
+    }
+    else
+      strPeerPort = ntohs(saPeer.sin_port);
   }
+#ifdef HAVE_IPV6
+  if(m_iDomain == AF_INET6)
+  {
+    struct sockaddr_in6 saPeer;
+    socklen iNameLen = sizeof(saPeer);
+    int iRc = getpeername(m_hSocket, (struct sockaddr*)&saPeer, &iNameLen);
+    ASSERT(iNameLen == sizeof(saPeer));
+    if(iRc)
+    {
+      throw E_Socket("Unable to get port of the peer: " +
+                     GetErrorMsg(SYSTEM_NETWORK));
+    }
+    else
+      strPeerPort = ntohs(saPeer.sin6_port);
+  }
+#endif
   else
-    strPeerPort = ntohs(saPeer.sin_port);
-  
+  {
+    ASSERT(false);
+  }
+
   return strPeerPort;
 }
 
