@@ -2,7 +2,7 @@
 * telnet.cpp:
 *-------------------------------------------------------------------------------
 * (c)1999-2001 VideoLAN
-* $Id: telnet.cpp,v 1.2 2001/10/07 20:16:43 bozo Exp $
+* $Id: telnet.cpp,v 1.5 2002/03/25 14:49:33 bozo Exp $
 *
 * Authors: Benoit Steiner <benny@via.ecp.fr>
 *          Arnaud de Bossoreille de Ribou <bozo@via.ecp.fr>
@@ -64,7 +64,6 @@
 #define DO      253             // Please use option
 #define WONT    252             // I won't use option
 #define WILL    251             // I will use option
-#define EC      247             // Erase the current character  
 #define EL      248             // Erase the current line
 
 #define TELOPT_ECHO     1       // Echo
@@ -191,10 +190,9 @@ C_TelnetCmdLine::C_TelnetCmdLine() : m_cCurrent(m_cHistory.CreateIterator())
 //------------------------------------------------------------------------------
 // 
 //------------------------------------------------------------------------------
-C_Buffer<byte> C_TelnetCmdLine::Append(byte bByte)
+C_String C_TelnetCmdLine::Append(byte bByte)
 {
-  C_Buffer<byte> cResult(1);
-  cResult.SetSize(1);
+  C_String cResult;
 
   switch(bByte)
   {
@@ -203,7 +201,7 @@ C_Buffer<byte> C_TelnetCmdLine::Append(byte bByte)
       if(m_strLine.Length() > 0)
       {
         m_strLine = m_strLine.SubString(0, m_strLine.Length()-1);
-        cResult[0] = EC;
+        cResult += "\b\e[P";
       }
       break;
     }
@@ -212,10 +210,8 @@ C_Buffer<byte> C_TelnetCmdLine::Append(byte bByte)
       if(bByte != '\n' && bByte != '\0')
       {
         m_strLine += (char)bByte;
-        cResult[0] = bByte;
+        cResult += (char)bByte;
       }
-      else
-        cResult.SetSize(0);
     }
   }
   
@@ -547,7 +543,7 @@ void C_TelnetSession::HandleByte(byte bByte)
       if(iStatus == YES)
       {
         // Remote echo      
-        C_Buffer<byte> cRemoteCmdLine = m_cCmdLine.Append(bByte);
+        C_String cRemoteCmdLine = m_cCmdLine.Append(bByte);
         if(m_iPhase != PASSWD_PHASE)
           m_cStream << cRemoteCmdLine;
       }
@@ -714,19 +710,46 @@ int C_Telnet::Init()
   int iRc = NO_ERR;
 
   // Get config
-  C_String strAddr = pApp->GetSetting("Telnet.LocalAddress", "0.0.0.0");
-  C_String strPort = pApp->GetSetting("Telnet.LocalPort", "9999");
+  C_String strDomain = pApp->GetSetting("Telnet.Domain", "Inet4").ToLower();
 
-  // Init the socket
-  try
+  int iDomain;
+  C_String strDefaultHost;
+
+  if(strDomain == "inet4")
   {
-    C_ConnectionsHandler<C_TelnetSession>::Init(strAddr, strPort);
+    iDomain = AF_INET;
+    strDefaultHost = "0.0.0.0";
   }
-  catch(E_Exception e)
+#ifdef HAVE_IPV6
+  else if(strDomain == "inet6")
   {
-    Log(m_hLog, LOG_ERROR, "Telnet server initialisation failed:\n" + e.Dump());
+    iDomain = AF_INET6;
+    strDefaultHost = "0::0";
+  }
+#endif
+  else
+  {
+    Log(m_hLog, LOG_ERROR, "Unknown domain:\n" + strDomain);
     iRc = GEN_ERR;
   }
+
+  if(!iRc)
+  {
+    C_String strAddr = pApp->GetSetting("Telnet.LocalAddress", strDefaultHost);
+    C_String strPort = pApp->GetSetting("Telnet.LocalPort", "9999");
+
+    // Init the socket
+    try
+    {
+      C_ConnectionsHandler<C_TelnetSession>::Init(iDomain, strAddr, strPort);
+    }
+    catch(E_Exception e)
+    {
+      Log(m_hLog, LOG_ERROR,
+          "Telnet server initialisation failed:\n" + e.Dump());
+      iRc = GEN_ERR;
+    }
+  }
   
   if(!iRc)
     Log(m_hLog, LOG_NOTE, "Telnet server initialised");
